using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace FileFinder
{
    public static class FileUtilities
    {
        public static IEnumerable<FileInfo> FilesChanged(string rootDir, TimeSpan epoch)
        {
            IEnumerable<string> dirs = FileUtilities.FindSubDiresExcluding(rootDir,
                delegate(string s)
                {
                    return FileUtilities.ExcludeDeveloperFiles(s);
                });

            IEnumerable<FileInfo> allFiles = FileUtilities.FilesInDirs(dirs);

            return FileUtilities.FileModifiedSince(allFiles, epoch);
        }

        public static IEnumerable<string> FindSubDiresExcluding(string dir, Predicate<string> removeIf)
        {
            string[] subDirs = null;
            try
            {
                subDirs = Directory.GetDirectories(dir);
            }
            catch (UnauthorizedAccessException)
            {
                // Just eat it, because we can't read this directory
                yield break;
            }
            foreach (string subDir in subDirs)
            {
                if (!removeIf(subDir))
                {
                    yield return subDir;
                    // Depth search:
                    foreach (string childDir in FindSubDiresExcluding(subDir, removeIf))
                        yield return childDir;
                }
            }
        }

        public static bool ExcludeDeveloperFiles(string filePath)
        {
            if (filePath.EndsWith("Debug"))
                return true;
            if (filePath.EndsWith("Release"))
                return true;
            if (filePath.EndsWith("obj"))
                return true;
            if (filePath.EndsWith(".svn"))
                return true;
            return false;
        }

        public static IEnumerable<FileInfo> FilesInDirs(IEnumerable<string> dirs)
        {
            foreach (string dir in dirs)
            {
                string[] allFiles = null;
                try
                {
                    allFiles = Directory.GetFiles(dir);
                }
                catch (UnauthorizedAccessException)
                {
                    // Eat it.
                }
                if (allFiles != null)
                    foreach (string file in allFiles)
                        yield return new FileInfo(file);
            }
        }

        public static IEnumerable<FileInfo> FileModifiedSince(IEnumerable<FileInfo> fileList,
            TimeSpan elapsed)
        {
            DateTime epoch = DateTime.Now - elapsed;
            return FileFilter(fileList, delegate(FileInfo thisFile)
                {
                    return thisFile.LastWriteTime > epoch;
                }
                );
        }

        public static IEnumerable<T> FileFilter<T>(IEnumerable<T> fileList,
            Predicate<T> func)
        {
            foreach (T singleFile in fileList)
            {
                if (func(singleFile))
                    yield return singleFile;
            }
        }

    }
}
